{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body %}
<div class="row">
  <p class="col-md-12">{% trans 'Confirm the current replica is to be promoted as the new replica source.' %}</p>
  <p class="col-md-12">{% trans 'This action cannot be undone.' %}</p>
  <fieldset>
  {% include "horizon/common/_form_fields.html" %}
  </fieldset>
</div>
<div class="row">
    <div class="left">
        <div class="well">
            <h4>{% trans "Current Replica" %}</h4>
            <table class='table table-bordered'>
              <tr><th>{% trans 'Name' %}</th><td>{{ replica.name|default:_("None") }}</td></tr>
              <tr><th>{% trans 'Host' %}</th><td>{{ replica.ip|default:_("Unknown") }}</td></tr>
              <tr><th>{% trans 'Status' %}</th><td>{{ replica.status|default:_("Unknown") }}</td></tr>
            </table>
        </div>
    </div>
    <div class="right">
        <div class="well">
            <h4>{% trans "Current Replica Source" %}</h4>
            <table class='table table-bordered'>
              <tr><th>{% trans 'Name' %}</th><td>{{ replica_source.name|default:_("None") }}</td></tr>
              <tr><th>{% trans 'Host' %}</th><td>{{ replica_source.ip|default:_("Unknown") }}</td></tr>
              <tr><th>{% trans 'Status' %}</th><td>{{ replica_source.status|default:_("Unknown") }}</td></tr>
            </table>
        </div>
    </div>
</div>
{% endblock %}
