{% extends "project/database_clusters/_detail_overview.html" %}
{% load i18n sizeformat %}

{% block connection_info %}
    <h4>{% trans "Connection Information" %}</h4>
    <hr class="header_rule">
    <dl class="dl-horizontal">
        {% with cluster.ip.0 as ip %}
            <dt>{% trans "Host" %}</dt>
            {% if not ip %}
                <dd>{% trans "Not Assigned" %}</dd>
            {% else %}
                <dd>{{ ip }}</dd>
                <dt>{% trans "Connection Examples" %}</dt>
                <dd>mysql -h {{ ip }} -u {% trans "USERNAME" %} -p</dd>
                <dd>mysql://{% trans "USERNAME" %}:{% trans "PASSWORD" %}@{{ ip }}:3306/{% trans "DATABASE"%}</dd>
            {% endif %}
        {% endwith %}
    </dl>
{% endblock %}
