{% load i18n %}

<p>{% blocktrans trimmed %}Specify the details for launching an instance.{% endblocktrans %}</p>
<p>{% blocktrans trimmed %}<strong>Please note:</strong> The value specified in the Volume Size field should be greater than 0, however, some configurations do not support specifying volume size. If specifying the volume size results in an error stating volume support is not enabled, enter 0.{% endblocktrans %}</p>
