{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <p>{% trans "Select a parameter and provide a value for the configuration parameter." %}</p>
{% endblock %}
