{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <p>{% trans "Specify the name of the new database." %}</p>
  <p>{% trans "Optionally provide a character set and collation for the database." %}</p>
{% endblock %}

