{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Log Contents" %}{% endblock %}

{% block main %}
<div class="row-fluid">
  <div class="col-sm-12">
  {% include "project/databases/logs/_log_contents.html" %}
  </div>
</div>
{% endblock %}