{% load i18n sizeformat %}

<div class="detail">
    <dl class="dl-horizontal">
        <dt>{% trans "Name" %}</dt>
        <dd>{{ cluster.name }}</dd>
        <dt>{% trans "ID" %}</dt>
        <dd>{{ cluster.id }}</dd>
        <dt>{% trans "Datastore" %}</dt>
        <dd>{{ cluster.datastore.type }}</dd>
        <dt>{% trans "Datastore Version" %}</dt>
        <dd>{{ cluster.datastore.version }}</dd>
        <dt>{% trans "Current Task" %}</dt>
        <dd>{{ cluster.task.name|title }}</dd>
        <dt>{% trans "RAM" %}</dt>
        <dd>{{ cluster.full_flavor.ram|mbformat }}</dd>
        <dt>{% trans "Number of Instances" %}</dt>
        <dd>{{ cluster.num_instances }}</dd>
        {% if cluster.locality %}
            <dt>{% trans "Locality" %}</dt>
            <dd>{{ cluster.locality }}</dd>
        {% endif %}
    </dl>

{% block connection_info %}
{% endblock %}
</div>
